pre {
  background: #fafafa;
  padding: 10px;
  white-space: pre-wrap;
}
div.stats {
  border: solid 1px #777;
  border-radius: 4px;
  box-shadow: 0 1px 1px rgba(0, 0, 0, .05);
  color: #777;
}
div.stats .stats-description {
  font-size: 0.8em;
  padding: 0 15px 15px;
  text-align: center;
}
div.stats .stats-body {
  font-size: 22px;
  padding: 15px 15px 0;
  text-align: center;
  word-break: break-all;
  word-wrap: break-word;
}
.table thead tr th {
  border-top: 0 !important;
}
.table th.fit, table td.fit {
  padding-left: 0px !important;
  width: 1%;
  white-space: nowrap;
}
.table.borderless td, .table.borderless th {
  border: none;
}
.noleftpad {
  padding-left: 0 !important;
}
.novertpad th, .novertpad td {
  padding-bottom: 0 !important;
  padding-top: 0 !important;
}
.wrappable {
  word-break: break-all;
  word-wrap: break-word;
}
